<?php

/*
The MIT License

Copyright (c) 2007 Tiago Bastos ; 2009 Dave Roberts

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

define('__DEBUG__',false);
define('__404_controller__', 'basics');
define('__404_action__', 'notfound');
define('__404__', '404.html');  // this is only used if you don't have __404_controller__ or __404_action__ defined
include_dir(DOCROOT.'/controllers/*.php');
function include_dir($pattern)
{
  ob_start();
  foreach (glob($pattern) as $file)
  {
    include_once($file);
  }
  ob_end_clean();
}

function Run()
{
  try { HappyPuppy::getInstance()->dispatch(); } catch (Exception $e)
  {
	header('HTTP/1.1 500 Internal Server Error');
    if (__DEBUG__==true) {
?><!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
  <head>
    <title>Error!</title>
  </head>
  <body>
    <h1>Caught exception: <?php print $e->getMessage(); ?></h1>
    <h2>File: <?php print $e->getFile()?></h2>
    <h2>Line: <?php print $e->getLine()?></h2>
    <h3>Trace</h3>
    <pre><?php print_r ($e->getTraceAsString()); ?></pre>
    <h3>Exception Object</h3>
    <pre><?php print_r ($e); ?></pre>
    <h3>Var Dump</h3>
    <pre><?php debug_print_backtrace (); ?></pre>
  </body>
</html><?php
    } else { not_found(); }
  }
}

function R($pattern, $controller, $action, $responds_to='', $http_method='GET'){
  HappyPuppy::getInstance()->add_url($pattern, $controller, $action, $responds_to, $http_method);
}

class HappyPuppy
{
  var $routes = array();
  static private $instance = NULL; 

  function __construct()
  {
    if (isset($_GET['url']))
      $this->url =trim( $_GET['url'], '/');
    else $this->url = '';
  }

  /* Add url to routes */
  public function add_url($rule, $klass, $klass_method, $responds_to = '', $http_method = 'GET')
  {
    $this->routes[] = array('/^' . str_replace('/','\/',$rule) . '$/', $klass,$klass_method,$responds_to,$http_method);
  }
  
  /* Process requests and dispatch */
  public function dispatch()
  {
    $found = false;
    $controller = ''; $action = ''; $responds_to = ''; $args = '';
    list($found, $controller, $action, $responds_to, $args) = $this->processRoutes($this->url, $this->routes);
    if (!$found)
    {
      list($controller, $action, $responds_to, $args) = $this->parseURL($this->url);
    }
	$this->RunController($controller, $action, $responds_to, $args);
  }
  public function RunController($controller, $action, $responds_to = '', $args = '')
  {
    $php_action = $this->getPHPAction($action);
    $controller_name = $controller."Controller";
	if (!class_exists($controller_name)){ if (!__DEBUG__){not_found();} else{ print("Can't find controller: ".$controller_name.".  Make sure Happy Puppy can see your controllers.  Happy Puppy by default loads all files under DOCROOT/controllers/"); exit(); } }
	$klass = new $controller_name();
    $klass->responds_to = $responds_to;
    $this->run_before_filters($klass, $action);
    ob_start();
    call_user_func_array(array($klass , $php_action),$args);
    $out = ob_get_contents();
    ob_end_clean();
    // did the page push out text?
    if ($klass->text_only)
    {
      print($out);
      exit();
    }
    // if not, we need to collect class variables, package them and process here
    $out = $klass->process($controller, $action);
    print($out);
  }
  
  private function processRoutes($url, $routes)
  {
    foreach($routes as $rule=>$conf)
    {
      // if we have a match
      if (preg_match($conf[0], $url, $matches) and $_SERVER['REQUEST_METHOD'] == $conf[4])
      {
        $args = $this->parse_urls_args($matches); //Only declared variables in url regex
        $controller = $conf[1]; $action = $conf[2]; $responds_to = $conf[3];
        return array(true, $controller, $action, $responds_to, $args);
      }
    }
    return array(false, null, null, null, null);
  }
  
  // Find the controller, action, responds to and args from a default URL
  private function parseURL($url)
  {
    // not concerned abouf anything after '?'
    $q_pieces = explode("?", $url);
    $half_1 = array_shift($q_pieces);
    $half_2 = implode("?", $q_pieces);
    // break off "responds_to" piece
    $dot_pieces = explode(".", $half_1);
    $half_1 = array_shift($dot_pieces);
    $responds_to = implode(".", $dot_pieces);
    $url = ($half_2 == "") ? $half_1 : $half_1.'?'.$half_2;
    $url_pieces = explode("/", $url);
    $controller = array_shift($url_pieces);
    $action = array_shift($url_pieces);
    $args = $url_pieces;
    return array($controller, $action, $responds_to, $args);
  }
  
  private function getPHPAction($action)
  {
    $phpaction = $action;
    if (function_exists($phpaction)) { $phpaction = "_".$phpaction; }
    if ($phpaction == 'list') { $phpaction = '_list'; }
    if ($phpaction == 'new') { $phpaction = '_new'; }
    if ($phpaction == '') { $phpaction = 'index'; }
    return $phpaction;
  }

  private function run_before_filters($klass, $action)
  {
    $this->run_filters($klass, $action, $klass->before, $klass->not_before);
  }
  private function run_filters($klass, $action, $filters, $unfiltered)
  {
    // Determine which filters to run
    $to_run = array();
    if ($filters != null) {
    foreach($filters as $filtered_action=>$filtered_methods)
    {
      if ($filtered_methods == "*"){ array_push($to_run, $filtered_action); continue; }
      foreach($filtered_methods as $method)
      {
        if ($method == $action){ array_push($to_run, $filtered_action); continue; }
      }
    } }
    if ($unfiltered != null) {
    foreach($unfiltered as $filtered_action=>$filtered_methods)
    {
      foreach($filtered_methods as $method)
      {
        if ($method == $action){ unset($to_run[array_search($method, $to_run)]); continue; }
      }
    } }
    foreach($to_run as $filtered_action)
    {
      call_user_func(array($klass , $filtered_action));
    }
  }

  private function parse_urls_args($matches)
  {
    $first = array_shift($matches);
    $new_matches = array();
    foreach($matches as $k=>$match){
      if (is_string($k)){
        $new_matches[$k]=$match;
      }
    }
    return $new_matches;
  }

  /* Singleton */
  public function getInstance()
  {
    if(self::$instance == NULL)
    {
      self::$instance = new HappyPuppy();
    }
    return self::$instance;
  }
}

function render_arr($file, $vars)
{
  $file = DOCROOT.'/views/'.$file.'.php';
  return Render::file_with_arr($file, $vars);
}

function render($file, $varname, $var)
{
  $file = DOCROOT.'/views/'.$file.'.php';
  return Render::file_with_var($file, $varname, $var);
}

class Render
{
  static function file_with_arr($file, $arr)
  {
    ob_start();
    foreach($arr as $key=>$value)
    {
      $$key = $value;
    }
    require($file);
    $out = ob_get_contents();
    ob_end_clean();
    return $out;
  }
  static function file_with_obj($file, $obj)
  {
    $vars = get_object_vars($obj);
    ob_start();
    if (!file_exists($file))
    {
      throw new Exception('View ['.$file.'] Not Found');
    }
    if (count($vars)>0)
    {
      foreach($vars as $key => $value)
      {
        $$key = $value;
      }
    }
    require($file);
    $out = ob_get_contents();
    ob_end_clean();
    return $out;
  }

  static function file_with_var($file, $varname, $var)
  {
    ob_start();
    $$varname = $var;
    require($file);
    $out = ob_get_contents();
    ob_end_clean();
    return $out;
  }

  static function file($file)
  {
    return Render::file_with_var($file, null, null);
  }
}

class C
{
  var $layout = true;
  var $layout_template = 'views/layout.php';
  var $view_template = '';
  var $view_header = '';
  var $text_only = false;
  var $responds_to = '';

  // this is called by Happy Puppy, not you
  public function process($controller, $action)
  {
    if ($action == null || $action == ''){ $action = 'index'; }
    $view_template = $this->view_template;
    if ($view_template == '') { $view_template = 'views/'.$controller.'/'.$action.'.php'; }
	if (!file_exists($view_template) AND !__DEBUG__){ not_found(); }
    if ($this->layout)
    {
      $content = Render::file_with_obj($view_template, $this);
      $head_template = 'views/'.$controller.'/'.$action.'.head.php';
      $head = "";
      if (file_exists($head_template))
      {
        $head = Render::file_with_obj($head_template, $this);
      }
	  $this->content = $content;
	  $this->head = $head;
      return Render::file_with_obj($this->layout_template, $this);
    }
    else
    {
      return Render::file_with_obj($view_template, $this);
    }
  }

  // These are helper methods called by you
  public function render_text($text = null)
  {
    if ($text != null){ echo $text; }
    $this->text_only = true;
  }
  public function render_no_layout($view_template = null)
  {
    $this->layout = false;
    if ($view_template != null){ $this->view_template = $view_template; }
  }
  public function render($layout_template = null, $view_template = null)
  {
    if ($layout_template != null){ $this->layout_template = $layout_template; }
    if ($view_template != null){ $this->view_template = $view_template; }
  }
  public function redirect($url)
  {
    header("Location: {$url}");
    exit();
  } 
}

function not_found()
{
	if (__404_controller__ != "" AND __404_action__ != "")
	{
		HappyPuppy::getInstance()->RunController(__404_controller__, __404_action__);
	}
	else
	{
		include(__404__);
	}
	exit();
}

?>
